function [F,H,dH,h,hdir] = vectorfields2s(t,y,params,str)

%Parameters
km     = params(1);
ktheta = params(2);
Ju     = params(3);
Jl     = params(4);
beta1  = params(5);
beta2  = params(6);
bup    = params(7);
bun    = params(8);
bl     = params(9);
Tsup   = params(10);
Tsun   = params(11);
Tsl    = params(12);
T1     = params(13);
T2     = params(14);
u      = params(15);

% Switching Manifold (H = [H1, H2])
H1 = y(2);
H2 = y(3);

% The vector normal to the switching manifold dH = grad(H);
dH1 = [0,1,0];
dH2 = [0,0,1];

% Internal functions
Tfu_pos  = 1/Ju*(Tsup+bup*y(2));
Tfu_neg  = 1/Ju*(-Tsun+bun*y(2));
Tfl_pos  = 1/Jl*( (Tsl+T1*(1-2/(1+exp(beta1*y(3))))+T2*(1-2/(1+exp(beta2*y(3)))))+bl*y(3));  
Tfl_neg  = 1/Jl*(-(Tsl+T1*(1-2/(1+exp(-beta1*y(3))))+T2*(1-2/(1+exp(-beta2*y(3)))))+bl*y(3));

% The general vector field 
F = [y(2)-y(3);...
     km/Ju*u-ktheta/Ju*y(1)-Tfu_pos;...
     ktheta/Jl*y(1)-Tfl_pos];

% Additional to the base vector field as H1 < 0

Delta1 = [0;...
          Tfu_pos-Tfu_neg;
          0];


% Additional to the base vector field as H2 < 0

Delta2 = [0;...
          0;
          Tfl_pos-Tfl_neg];


% Poincare section
% Change the Poicare section h1 below and the desired action in
% psection2s.m
h = [1,1]; 

% Direction of locations of Poincare sections
hdir = [1,1];

% Assembly of vectors

F = [F,Delta1,Delta2];
H = [H1,H2];
dH = [dH1;dH2];
